/*
 * mp3plot - Bitrate analysis tool
 *
 * Copyright (C) 2007 Toni Corvera
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

// $Id: mp3_common.h 723 2007-05-20 13:29:19Z  $

#ifndef ONET_MP3COMMON_H
#define ONET_MP3COMMON_H

#include <string>

#include <sysexits.h>

namespace net_outlyer {

namespace mp3 {

// Used to call read functions in a more compact way
#define CHARR(a) (reinterpret_cast<char *>((a)))

    // Extern declarations for the modules that include this file
    extern const uint16_t BAD_BITRATE;
    extern const uint8_t BITRATE_TABLE_LEN;
    extern const uint16_t BITRATE_TABLE[];

    // Generic error
    class e_error {
        private:
            std::string msg;
        public:
            const int error_code;

            e_error(const std::string & s, int errcode = EX_SOFTWARE);
            e_error()
                : msg("Undefined internal error"), error_code(EX_SOFTWARE)
            {}

            const std::string & get_msg() const;
    };

    class e_read_failure : public e_error {
        public:
            e_read_failure(const std::string & msg, int errcode = EX_IOERR)
                : e_error(msg, errcode) {}

            e_read_failure()
                : e_error("Failed to read data.", EX_IOERR) {}
    };

    struct e_bad_filetype : public e_error {
        public:
            e_bad_filetype();
        protected:
            e_bad_filetype(const std::string & m, const int errcode = EX_DATAERR)
                : e_error(m, errcode)
            {}
    };

    // The values match the bit in the header
    enum mpeg_version_t {
        MPEG2_5 = 0x00,
        _version_reserved = 0x01,
        MPEG2 = 0x02,
        MPEG1 = 0x03,
    };

    // The values match the bit in the header
    enum mpeg_layer_t {
        _layer_reserved = 0,
        LAYER_III,
        LAYER_II,
        LAYER_I,
    };

    // The values match the bit in the header
    enum channel_mode_t {
        STEREO = 0,
        JOINT_STEREO = 1,
        DUAL = 2,
        DUAL_MONO = DUAL,
        MONO = 3,

        _undefined_channel_mode = 5, // this one is not part of the spec
    };

    enum vbr_flags_t {
        // Set if number of frames is stored
        FL_NUMFRAMES = 0x01,
        // Set if filesize (in bytes) is stored
        FL_FILESIZE  = 0x02,
        // Set if TOC is stored
        FL_TOC       = 0x04,
        // Set if VBR scale is stored
        FL_VBR_SCALE = 0x08,
    };

    // Variants of mp3 files
    enum mp3_filetypes_t {
        FT_MP3,
        FT_ID3V2,
        UNKNOWN,
    };


} // namespace mp3

} // namespace net_outlyer

#endif // ONET_MP3COMMON_H
/* vim:set ts=4 et ai: */

